<template>
  <div class="table-page">
    <div class="table-page-content">
      <div class="left-tree">
        <tree-side
          ref="treeSide"
          :dataLoading="treeLoading"
          :default-expand-all="false"
          :defaultProps="defaultProps"
          :tabList="['组织路段树']"
          :treeData="treeData"
          isDeselect
          @currentChange="currentChange"
        ></tree-side>
      </div>
      <div class="right-table">
        <div class="full-page bg">
          <div class="search-box">
            <div class="search-box-combination">
              <SearchCommon
                :optionList="optionList"
                @search="handleSearch"
              />
            </div>
          </div>
          <div class="main-table-container">
            <div class="table-container" style="padding: 20px">
              <div ref="btnPanel" class="btn-box">
                <el-button
                  :loading="syncLoading"
                  class="btn btn-plus"
                  icon="el-icon-receiving"
                  size="small"
                  type="primary"
                  @click="syncData"
                >
                  同步
                </el-button>
              </div>
              <div ref="tableRef" class="table-content">
                <el-table
                  ref="multipleSelection"
                  v-loading="tableLoading"
                  :data="tableData"
                  :default-sort="getSorterDirection(params,true)"
                  :max-height="tableMaxHeight"
                  border
                  class="table-list"
                  element-loading-background="rgba(10 ,38 ,61, 0.9)"
                  row-key="id"
                  stripe
                  @select="handleSelection"
                  @selection-change="handleSelectionChange"
                  @sort-change="sortChange"
                >
                  <el-table-column
                    :index="index => index + 1 + (params.pageBean.page - 1) * params.pageBean.pageSize"
                    align="center"
                    label="序号"
                    type="index"
                    width="75"
                  />
                  <el-table-column
                    :formatter="({number}) => number || '-'"
                    align="center"
                    label="编号"
                    min-width="110"
                    prop="number"
                    show-overflow-tooltip
                    sortable="custom"
                  />
                  <el-table-column
                    :formatter="({roadSegmentName}) => roadSegmentName || '-'"
                    align="center"
                    label="所在路段"
                    min-width="80"
                    prop="roadSegmentName"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({startPeg}) => startPeg || '-'"
                    align="center"
                    label="起点桩号"
                    min-width="80"
                    prop="startPeg"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({endPeg}) => endPeg || '-'"
                    align="center"
                    label="止点桩号"
                    min-width="80"
                    prop="endPeg"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({direction}) => dictionaryEscape('qlfx', direction) || '-'"
                    align="center"
                    label="边坡方向"
                    min-width="80"
                    prop="direction"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({protectType}) => protectType || '-'"
                    align="center"
                    label="保护类型"
                    min-width="90"
                    prop="protectType"
                    show-overflow-tooltip
                    sortable="custom"
                  />
                  <el-table-column
                    :formatter="({protectMaterial}) => protectMaterial || '-'"
                    align="center"
                    label="防护材料"
                    min-width="80"
                    prop="protectMaterial"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({height}) => height || '-'"
                    align="center"
                    label="高度"
                    min-width="70"
                    prop="height"
                    show-overflow-tooltip
                    sortable="custom"
                  />
                  <el-table-column
                    :formatter="({length}) => length || '-'"
                    align="center"
                    label="长度"
                    min-width="70"
                    prop="length"
                    show-overflow-tooltip
                    sortable="custom"
                  />
                  <el-table-column
                    :formatter="({isHighRisk}) => dictionaryEscape('bplx', isHighRisk) || '-'"
                    align="center"
                    label="边坡等级"
                    min-width="80"
                    prop="isHighRisk"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({sideType}) => dictionaryEscape('bpfl', sideType,{congruent:false}) || '-'"
                    align="center"
                    label="边坡类型"
                    min-width="80"
                    prop="sideType"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({isRamp}) => dictionaryEscape('sftyzd', isRamp,{congruent:false}) || '-'"
                    align="center"
                    label="是否斜坡"
                    min-width="80"
                    prop="isRamp"
                    show-overflow-tooltip
                  />
                  <el-table-column
                    :formatter="({rxcavateLevel}) => rxcavateLevel || '-'"
                    align="center"
                    label="开挖级数"
                    min-width="90"
                    prop="rxcavateLevel"
                    show-overflow-tooltip
                    sortable="custom"
                  />
                  <el-table-column
                    align="center"
                    label="操作"
                    min-width="65"
                    show-overflow-tooltip
                  >
                    <template v-slot="{row}">
                      <el-link
                        :underline="false"
                        type="primary"
                        @click="handleDetailPage(row)"
                      >
                        详情
                      </el-link>
                    </template>
                  </el-table-column>
                </el-table>
                <pagination
                  ref="paginationPanel"
                  :pageResult="params.pageBean"
                  style="margin-top: 10px"
                  @size-change="handleSizeChange"
                  @current-change="handleCurrentChange"
                />
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
</template>

<script>
import {getSorterDirection, refineRequestBody, setQueryValue} from '@/utils/requestWrappers'
import pagination from '@/components/SuperPagination/index.vue'
import {TableMixin} from '@/mixins/tableMixin'
import treeSide from '@/components/treeSide/index.vue'
import SearchCommon from '@/components/SearchCommon/index.vue'
import {synchroData, tableData} from '@/api/patrolAndRescue/infoMgmtPatrolsRescues/slopeInfo'
import PubSub from 'pubsub-js'
import {dictionaryEscape, getDictionaryByCode} from '@/utils/dictionary'
import {getTree} from '@/api/tree'

export default {
  name: 'TablePage',
  components: {
    pagination,
    treeSide,
    SearchCommon
  },
  mixins: [TableMixin],
  data() {
    return {
      pubSub: null,
      treeData: [], //左边树
      tableLoading: true,
      treeLoading: true,
      defaultProps: {
        children: 'children',
        label: 'name'
      },
      tableData: [],
      optionList: [
        {title: '编号', prop: 'number', type: 'input', default: true},
        // {title: '路段名称', prop: 'road_segment_id_', type: 'select', option: [], multiple: false},
        {title: '边坡方向', prop: 'direction', type: 'select', option: getDictionaryByCode('qlfx'), multiple: true},
        {title: '边坡等级', prop: 'isHighRisk', type: 'select', option: getDictionaryByCode('bplx'), multiple: true},
        {title: '边坡类型', prop: 'sideType', type: 'select', option: getDictionaryByCode('bpfl'), multiple: true},
        {title: '是否斜坡', prop: 'isRamp', type: 'select', option: getDictionaryByCode('sftyzd'), multiple: true},
        {title: '请输入开挖级数（大于）', prop: 'rxcavate_level_', type: 'input'}
      ],
      params: {
        pageBean: {pageSize: 20, page: 1, total: 0},
        querys: [
          {operation: 'IN', property: 'roadSegmentId'},
          {operation: 'LIKE', property: 'COMPANY_IDS_'},
          {operation: 'LIKE', property: 'number'},
          // {operation: 'EQUAL', property: 'road_segment_id_'},
          {operation: 'IN', property: 'direction'},
          {operation: 'IN', property: 'isHighRisk'},
          {operation: 'IN', property: 'sideType'},
          {operation: 'IN', property: 'isRamp'},
          {operation: 'GREAT', property: 'rxcavate_level_'}
        ],
        sorter: [
          {direction: '', property: 'number'},
          {direction: '', property: 'protectType'},
          {direction: '', property: 'height'},
          {direction: '', property: 'length'},
          {direction: '', property: 'rxcavateLevel'}
        ]
      },
      treeItem: {}, // 树形节点选中对象
      syncLoading: false // 同步按钮Loading
    }
  },
  created() {
    // 获取树形目录
    this.getOrgTreeData(treeData => {
      // 回调中默认选中第一项并将其id作为参数请求表格
      this.treeItem = treeData[0] || {}
      let array = []
      if (this.treeItem.children.length > 0) {
        array = this.extractIds(this.treeItem)
      }
      this.$nextTick(_ => {
        this.$refs.treeSide?.setCurrentKey(treeData[0]?.id)
        this.$refs.treeSide?.expandHandle(true, 2)
      })
      setQueryValue(this.params, 'roadSegmentId', array)
      // this.getRoadSectionInfo(this.treeItem?.path)  // 获取路段信息
      // setQueryValue(this.params, 'COMPANY_IDS_', this.treeItem?.path || '')

      this.loadData()
    })
  },
  mounted() {
    this.pubSub = PubSub.subscribe('TablePage', (_, data) => {
      switch (data.type) {
        case 'add':
          this.params.pageBean.page = 1
          this.loadData()
          break
        case 'edit':
          this.loadData()
          break
      }
    })

  },
  methods: {
    dictionaryEscape,
    getSorterDirection,
    extractIds(node) {
      let ids = []
      // 如果当前节点有 children，则递归提取
      if (node.children) {
        ids = node.children.map(child => child.id) // 提取当前节点的 id
        // 递归提取子节点的 id
        node.children.forEach(child => {
          ids = ids.concat(this.extractIds(child))
        })
      }
      return ids
    },
    // 获取公司树
    async getOrgTreeData(callback) {
      try {
        this.treeLoading = true
        let {value} = await getTree()
        this.treeData = [value]
      } finally {
        this.treeLoading = false
        callback && callback(this.treeData)
      }
    },
    transformTreeData(data) {
      return {
        name: data.name,
        id: data.id,
        children: data.children.reduce((acc, child) => {
          acc.push({
            id: child.id,
            name: child.name,
            path: child.path,
            children: child.roadList.map(road => ({
              id: road.id,
              name: road.name,
              path: road.path
            }))
          })
          return acc
        }, [])
      }
    },
    // 获取路线信息
    // async getRoadSectionInfo(value) {
    //   const {rows = []} = await roadSectionInfo(refineRequestBody({
    //     pageBean: {pageSize: -1},
    //     querys: [{operation: 'LIKE', property: 'COMPANY_IDS_', value}]
    //   }))
    //   this.optionList[1].option = rows.map(i => ({...i, value: i.id}))
    // },

    //获取数据
    async loadData() {
      try {
        this.tableLoading = true
        const {total = 0, rows = []} = await tableData(refineRequestBody(this.params))
        this.params.pageBean.total = total
        this.tableData = rows
      } finally {
        this.tableLoading = false
      }
    },

    //左边树节点
    currentChange(nodeItem) {
      // this.getRoadSectionInfo(nodeItem?.path) // 获取路线信息
      if (nodeItem.id === '1766017468444450816') {
        setQueryValue(this.params, 'roadSegmentId ', this.extractIds(nodeItem))
      } else if (nodeItem.children && nodeItem.children.length > 0) {
        const array = nodeItem.children.map(item => item.id)
        setQueryValue(this.params, 'roadSegmentId ', array)
      } else {
        setQueryValue(this.params, 'roadSegmentId ', [nodeItem?.id] || [])
      }
      // if (nodeItem.children && nodeItem.children.length > 0) {
      //   this.isAddDisabled = true // 禁用新增按钮
      //   const array = nodeItem.children.map(item => item.id)
      //   setQueryValue(this.params, 'roadSegmentId ', array)
      //   this.$message.warning('请选择二级目录进行新增操作')
      // } else {
      //   this.isAddDisabled = false // 启用新增按钮
      //   // 修改对应的字段值
      //   setQueryValue(this.params, 'roadSegmentId ', [nodeItem?.id] || [])
      // }
      // 修改对应的字段值
      // setQueryValue(this.params, 'COMPANY_IDS_', nodeItem?.path || '')
      this.treeItem = nodeItem || {}
      this.params.pageBean.page = 1
      this.loadData()
      this.calcTableHeight(500)
    },

    // 同步数据
    syncData() {
      this.$confirm('是否同步边坡数据？', '提示', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(async () => {
        try {
          this.syncLoading = true
          const {state} = await synchroData()
          if (state) {
            this.$message.success(`边坡数据同步成功`)
            this.params.pageBean.page = 1
            await this.loadData()
          }
        } finally {
          this.syncLoading = false
        }
      }).catch(_ => _)
    },

    // 改变页面
    changePage(pageName = '', row = {}) {
      if (!pageName) return

      this.$emit('changePage', pageName, _ => {
        PubSub.publish(pageName, row)
      }, row.id ? '编辑' : '添加')
    }
  },
  beforeDestroy() {
    // 在组件销毁之前 清除订阅消息
    PubSub.unsubscribe(this.pubSub)
  }
}
</script>

<style lang="scss" scoped>
.table-page {
  ::v-deep .left-tree {
    width: 275px !important;
  }
}
</style>
